/* An comment. */
@charset 'UTF-8';
@import url('foo-external.css');
/* Another comment. */
@import url('/baz/vim/other external.css');
@import url('\'escaped\'quotes\\\'arecool.css');

/**
 * Fun comments.
 */
:root {
  --some-var: red;
  --some-mixin: {
    color: red;
  };

  --another-mixin: {
    /* Blue! */
    color: blue;
  };
}

p {
  color: red;
}

p.mixed-in {
  @apply(--some-mixin);
}

; /* Random semicolon. */

.some-class {
  text-decoration: underline;
  color: var(--some-var);

  pathological-property: no-semicolon
}

@keyframes AnAnimation {
  0% {
    top: 0px;
  }

  100% {
    top: 10px;
  }
}

/*
 * #pathological.comment:foo ***/
div.interesting > .combinator:not(:nth-of-type(0)) {
  font-family: 'Missing Semicolon'
  position: relative;
}

#AnID {
  /* Nice comment! */
  display: block;
}

@media screen (min-width > 1024px) {
  p {
    color: blue;
  }
}


/** Minified! **/
@font-face{font-family:foobar-icons;src:url(http://www.i.cdn.foobar.com/.a/fonts/icons/2.0.6/foobar-icons.eot);src:url(http://www.i.cdn.foobar.com/.a/fonts/icons/2.0.6/foobar-icons.eot?#iefix) format("embedded-opentype"),url(http://www.i.cdn.foobar.com/.a/fonts/icons/2.0.6/foobar-icons.woff) format("woff"),url(http://www.i.cdn.foobar.com/.a/fonts/icons/2.0.6/foobar-icons.ttf) format("truetype"),url(http://www.i.cdn.foobar.com/.a/fonts/icons/2.0.6/foobar-icons.svg?#foobar-icons) format("svg");font-weight:400;font-style:normal}.ad-slide__next:after,.ad-slide__prev:after,.el-button--follow:before,.el-button--share:before,.el-button--upload:before,.el-drop-down:after,.el-drop-down__arrow-up:after,.el-timestamp:before,.el__gallery-caption--closed:after,.el__gallery-caption:after,.el__storyelements--close:before,.el__video--expandable .media__video--demand:before,.el__video--standard .media__video--demand:before,.el__video__play-button:before,.el__video__play-button:hover:before,.el__video_collection--close:before,.icon,.icon-media-audio:before,.icon-media-gallery:before,.icon-media-games:before,.icon-media-image:before,.icon-media-infographic:before,.icon-media-map:before,.icon-media-video:before,.icon-off:before,.icon-pause:before,.icon-play:before,.icon-repeat:before,.icon-sound:before,.icon-stop:before,.owl-next:before,.owl-prev:before,.pg .video__description-toggle.hide-desc:before,.pg .video__description-toggle:before,.pg-wrapper .form .select:after,.pg-wrapper form .select:after,.pullquote>blockquote:before{-moz-osx-font-smoothing:grayscale;-webkit-font-smoothing:antialiased;display:inline-block;font-family:foobar-icons;font-style:normal;font-weight:400;line-height:1;speak:none;text-decoration:inherit;text-rendering:optimizeLegibility;text-transform:none;vertical-align:middle}.el-button,b,strong{font-weight:700}.el-button,a,h1 a:hover,h2 a:hover,h3 a:hover,h4 a:hover,h5 a:hover,h6 a:hover{text-decoration:none}.icon--arrow-down:before{content:"\f101"}.icon--arrow-left:before{content:"\f102"}.icon--arrow-line-down:before{content:"\f103"}.icon--arrow-line-up:before{content:"\f104"}

.has-a-data-url {
  background: url(data:image/svg+xml;base64,PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0idXRmLTgiPz4gPHN2ZyB2ZXJzaW9uPSIxLjEiIHhtbG5zPSJodHRwOi8vd3d3LnczLm9yZy8yMDAwL3N2ZyI+PGRlZnM+PGxpbmVhckdyYWRpZW50IGlkPSJncmFkIiBncmFkaWVudFVuaXRzPSJvYmplY3RCb3VuZGluZ0JveCIgeDE9+IA==), none 0 0;
}
