/**
@license
Copyright (c) 2016 The Polymer Project Authors. All rights reserved.
This code may only be used under the BSD style license found at http://polymer.github.io/LICENSE.txt
The complete set of authors may be found at http://polymer.github.io/AUTHORS.txt
The complete set of contributors may be found at http://polymer.github.io/CONTRIBUTORS.txt
Code distributed by Google as part of the polymer project is also
subject to an additional IP rights grant found at http://polymer.github.io/PATENTS.txt
*/

/* Exported from Chrome 52 on OS X */
exports.initialValues = {
  "animation": "none 0s ease 0s 1 normal none running",
  "background": "rgba(0, 0, 0, 0) none repeat scroll 0% 0% / auto padding-box border-box",
  "border": "0px none rgb(0, 0, 0)",
  "bottom": "auto",
  "clear": "none",
  "clip": "auto",
  "color": "rgb(0, 0, 0)",
  "columns": "auto auto",
  "contain": "none",
  "cursor": "auto",
  "cx": "0px",
  "cy": "0px",
  "d": "none",
  "direction": "ltr",
  "display": "inline",
  "fill": "rgb(0, 0, 0)",
  "filter": "none",
  "flex": "0 1 auto",
  "float": "none",
  "font": "normal normal normal normal 16px / normal Times",
  "height": "auto",
  "isolation": "auto",
  "left": "auto",
  "margin": "0px",
  "mask": "none",
  "motion": "none 0px auto 0deg",
  "opacity": "1",
  "order": "0",
  "orphans": "2",
  "outline": "rgb(0, 0, 0) none 0px",
  "overflow": "visible",
  "padding": "0px",
  "perspective": "none",
  "position": "static",
  "r": "0px",
  "resize": "none",
  "right": "auto",
  "rx": "0px",
  "ry": "0px",
  "speak": "normal",
  "stroke": "none",
  "top": "auto",
  "transform": "none",
  "transition": "all 0s ease 0s",
  "visibility": "visible",
  "widows": "2",
  "width": "auto",
  "x": "0px",
  "y": "0px",
  "zoom": "1"
};
